pro QabsCalc, grainsize, lambda, Qabs, lambdaQabs, Qabsreturn, a
; This program reads in Qabs as calcualted by the DUSTY 
; program by Ivezic et al.  Its input parameters are the 
; user specified grain size and wavelength (both in microns), 
; and it returns Qabs

; Written 7/02  by

; Joannah Metz
; Harvard-Smithsonian Center for Astrophysics
; Mail Stop #10
; 60 Garden St. 
; Cambridge, MA 02138

lambda0=lambda*1e-4

; The dust sizes are stored in a, wavelengths in lambdaQabs, and the absorption 
; coefficients in Qabs

index=-1

; Qabs is only calculated for grainsizes larger than 0.01 microns and smaller 
; than 3.0 microns, if the user supplied grainsize is smaller than 0.01, 
; the program uses the Qabs for 0.01.  If the grainsize is larger than 3 
; microns, it is approximated as a power law.  This subroutine searches a list
; of precalculated absorption coefficients for 21 different grain sizes and 112
; different wavelengths.  If the specified grain size and wavelength are not on
; the list then it interpolates between the next lowest and highest value.


IF ((grainsize GE 0.01) AND (grainsize LE 3)) THEN BEGIN
  index=where(a EQ grainsize)
  IF (ARRAY_EQUAL(index,-1)) THEN BEGIN
    uindex=where(a GT grainsize)
    upperindex=uindex(0)
    lindex=where(a LT grainsize)
    lowerindex=size(lindex)
    lowerindex=lowerindex(1)-1 
  ENDIF 
ENDIF

IF ((lambda GE 0.01) AND (lambda LE 36000)) THEN BEGIN
  lambdaindex=where(lambda EQ lambdaQabs)
  IF (ARRAY_EQUAL(lambdaindex,-1)) THEN BEGIN
    uLambdaIndex=where(lambdaQabs GT lambda)
    upperLambdaIndex=uLambdaIndex(0)
    lLambdaIndex=where(lambdaQabs LT lambda)
    lowerLambdaIndex=size(lLambdaIndex)
    lowerLambdaIndex=lowerLambdaIndex(1)-1
    IF (index(0) EQ -1) THEN BEGIN
      f=[Qabs(lowerindex,lowerLambdaIndex),Qabs(upperindex,upperLambdaIndex)]
      Qabsreturn=interpolate(f,lambda)
    ENDIF ELSE BEGIN
      f=[Qabs(index(0),lowerLambdaIndex),Qabs(index(0),upperLambdaIndex)]
      Qabsreturn=interpolate(f,lambda)
    ENDELSE
  ENDIF ELSE BEGIN
    IF (index(0) EQ -1) THEN BEGIN
      f=[Qabs(lowerindex,lambdaindex),Qabs(upperindex,lambdaindex)]
      Qabsreturn=interpolate(f,lambda)
    ENDIF ELSE BEGIN
      Qabsreturn=Qabs(index(0),lambdaindex)
    ENDELSE
  ENDELSE
ENDIF ELSE BEGIN
  IF (lambda LE 0.01) THEN BEGIN
    lambdaindex=0
  ENDIF
  IF (lambda GE 36000) THEN BEGIN
    lambdaindex=111
  ENDIF
ENDELSE


IF ((lambda LE 0.01) OR (lambda GE 36000)) THEN BEGIN
  IF (index(0) EQ -1) THEN BEGIN
    IF (grainsize LT 0.01) THEN BEGIN
      Qabsreturn=Qabs(0,lambdaindex)
    ENDIF ELSE BEGIN
      f=[Qabs(lowerindex,lambdaindex),Qabs(upperindex,lambdaindex)]
      Qabsreturn=interpolate(f,lambda)
    ENDELSE
  ENDIF ELSE BEGIN
     Qabsreturn=Qabs(index(0),lambdaindex)
  ENDELSE
ENDIF
    

end
