pro xusermapzodimodel, lstar, tstar, rstar, num, inu, stepau, $
                       lambda, radin, pfunc500, im, Qabsuser, iras=iras, $
                       scatterflag=scatterflag, useralpha=useralpha, $
                       userdelta=userdelta, userdustsize=userdustsize, $
                       lambdaQabs = lambdaQabs, emit = emit, $
                       scaletoflux = scaletoflux

; Adapted from xfullzodimodel during 06/02 by

; Sean Moran
; Harvard-Smithsonian Center for Astrophysics
; 60 Garden St. 
; Cambridge, MA 02138

; the x is for expansion
; this code is for making big big images

; lstar luminosity of the star in solar luminosities
; num is the number of steps in radius to take
; tstar and rstar are the temperature and radius of the star

; inu will be an 2*num by 2*num array containing the emission map

; stepau is the stepsize in au

; radin and radout are the is the disk inner and outer radii in AU

; set iras to use IRAS model
; lambdaQabs and emit are passed through to temperaturecalc


;*****************************************************************
; fundamental constants
pi=3.1415926536
pi2=pi*2.0
kb=1.38066d-16
cc=2.9979d10
hp=6.62608d-27
sigma=5.6705d-5
l10=2.30259

alpha = 1.34
beta = 4.14
gamma = 0.942
delta=0.467
T0=286.0
mu=0.189
;<n sigma> in AU^-1 measured at the Earth
n0=1.13d-7
al=0.36  ; so Hong phase function agrees with DIRBE at 90 degrees 

if keyword_set(iras) then begin
alpha=1.803
beta=4.973
gamma=1.265
T0=266.20
delta=0.359
n0=2.1527e-7  ;   =1.439e-20*1.495979e13
endif

if useralpha NE 0 then alpha=useralpha
if userdelta NE 0 then delta=userdelta
;if albedo NE 0 then al=albedo

;************************************************************
; More Definitions

num2=num*2

;l0 is the wavelength, in cm
l0= lambda*1e-4

; some math we can do now to make the Bnu calculation faster 
chk=2.0*(kb^3.0)/((cc*hp)^2.0)
hnuok=hp*(cc/l0)/kb
hnuok_scale = hp*(cc/(scaletoflux[1]*1.0e-4))/kb

; If the dust size is less than 3 microns, then it takes the appropriate
; absorption coefficient and stores it to `em'

em=1

IF keyword_set(userdustsize) then begin
  IF (userdustsize GT 3) THEN BEGIN 
    lambda0=userdustsize*10
    em=(lambda0/lambda)^2 < 1.0
  ENDIF ELSE BEGIN
     em=Qabsuser(0)
  ENDELSE
ENDIF ELSE BEGIN
    em=(280.0/(280.0+lambda))+((2.2-0.45*lambda) > 0)
ENDELSE


; where we're gonna put the results
inu=fltarr(num2, num2)

; make a vector that contains bnu as a function of radius,
; sampled every 1/5 of a radial step

raus=(findgen(num*9.0)+0.5)*stepau/5.0 > 1e-8 
;gotta find temperatures of dust before we get Bnu
if keyword_set(userdustsize) then begin
print, 'Calculating the equilibrium temperature of the dust'
temperaturecalc, lstar, tstar, userdustsize, raus, t, lambdaQabs, emit
endif else begin
t = T0 * raus^(-delta)* (lstar^(delta/2.0))
ENDELSE

;create a cubic array containing distance from center, then
;temperature, then bnu at that position
print, 'generating emission map from the dust density map...'
x = (findgen(num)+0.5)*stepau
y = make_array(num, /float, value = 1.0)
plane = x#y
plane = plane^2.0+(transpose(plane))^2.0
plane=[[rotate(plane,2),rotate(plane,3)],[rotate(plane,1),plane]]
cube = make_array(num2, num2, num2, /float, value = 1.0)
FOR i = 0, num-1 DO BEGIN
  cube[num-i-1, *, *] =plane[*, *]+(stepau*(float(i)+0.5))^2.0
  cube[num+i, *, *] = cube[num-i-1, *, *]
ENDFOR
cube = sqrt(cube)

;clear density map within radin
indices = where(cube[(num-radin-3):(num+radin+2),(num-radin-3):(num+radin+2), (num-radin-3):(num+radin+2)] LT radin, cnt)
IF cnt NE 0 THEN im[indices] = 0.0

;fill with temperatures
cube[*, *, *]= t[floor(cube[*, *, *]/stepau*5.0)]

;scale the density map to match the known flux scaletoflux[0] at the
;wavelength scaletoflux[1]
IF (n_elements(scaletoflux) EQ 2) AND (scaletoflux[1] NE lambda) THEN BEGIN
  ;calculate the dust emission coeff at the scaleto wavelength
  print, 'Scaling to match ', scaletoflux[0], 'Jy at ', $
    scaletoflux[1], 'microns'
  em2=1.0
  IF keyword_set(userdustsize) then begin
    IF (userdustsize GT 3) THEN BEGIN 
      em2=(280.0/(280.0+scaletoflux[1]))+((2.2-0.45*scaletoflux[1]) > 0)
    ENDIF ELSE BEGIN
      em2=Qabsuser(0)
    ENDELSE
  ENDIF ELSE BEGIN
      em2=(280.0/(280.0+scaletoflux[1]))+((2.2-0.45*scaletoflux[1]) > 0)
  ENDELSE
  ;calculate the total emission
  flux = hnuok_scale/cube
  flux = (cube^3.0)*chk*flux^3.0/(exp(flux)-1.0)
  scalefactor = total(flux*im*em2)
  im = im*scaletoflux[0]/scalefactor
ENDIF

;Calculate Planck spectrum Bnu (erg s^-1 cm^-2 ster^-1 Hz^-1)
; for grains at each of the temperatures in t_cube
bnu=hnuok/cube
bnu=(cube^3.0)*chk*bnu^3.0/(exp(bnu)-1.0)

;multiply the density map by bnu and emission coefficient
im = bnu*im*em

;if scaling to  flux at the same wavelength as the image we're making,
;take this shortcut.
IF scaletoflux[1] EQ lambda THEN BEGIN
  im = im/total(im)*scaletoflux[0]
ENDIF

; Now integrate the emission along lines of sight
IF n_elements(scaletoflux) EQ 2 THEN inu = total(im, 3) ELSE $
   inu=stepau*total(im,3) 
;  We're done.  Go return the answer to zodipic.
print,  '...done'
end
